import axios from 'axios'
import handleApiError from '../handleApiError'

const state = {
  files: [],
  log: [],
  loaded: {
    files: false,
    log: false,
  }
}

const getters = {
  files: state => state.files,
  fileCount: state => state.files.length,
  log: state => state.log,
  getFileById: state => id => {
    let file
    if (id !== undefined) {
      file = state.files.find(f => f.id === id)
      if (file === undefined) {
        console.log('[ERROR] getFileById: ID not found in store!')
      }
    }
    return file
  }
}

const mutations = {
  loading(state, item) { state.loaded[item] = false },
  finishLoading(state, item) { state.loaded[item] = true },

  setFiles (state, files) { state.files = files },
  addFile (state, file) { state.files.push(file) },
  setFileMeta (state, data) {
    let file = state.files.find(f => f.id === data.id)
    file.metadata = data.metadata
  },

  setLog (state, log) { state.log = log },
}

const actions = {
  // Fetch all files for a given show from the AuRa tank API
  fetchFiles (ctx, data) {
    ctx.commit('loading', 'files')
    let uri = process.env.VUE_APP_API_TANK + 'shows/' + data.slug + '/files'
    axios.get(uri, {
      withCredentials: true,
      headers: { 'Authorization': 'Bearer ' + ctx.rootState.auth.user.access_token }
    }).then(response => {
      // we don't have to check separately, if there are files, because tank
      // always provides an empty array if there are no files (or even if there is no corresponding show)
      ctx.commit('setFiles', response.data.results)
      ctx.commit('finishLoading', 'files')
      if (data && typeof(data.callback) === 'function') { data.callback() }
    }).catch(error => {
      handleApiError(this, error, 'could not load files')
      if (data && typeof(data.callbackCancel) === 'function') { data.callbackCancel() }
    })
  },

  fetchLog (ctx, data) {
    ctx.commit('loading', 'log')
    let uri = process.env.VUE_APP_API_TANK + 'shows/' + data.slug + '/files/' + data.file + '/logs'
    axios.get(uri, {
      withCredentials: true,
      headers: { 'Authorization': 'Bearer ' + ctx.rootState.auth.user.access_token }
    }).then(response => {
      // we don't have to check separately, if there are files, because tank
      // always provides an empty array if there are no files (or even if there is no corresponding show)
      ctx.commit('setLog', response.data.results)
      ctx.commit('finishLoading', 'log')
      if (data && typeof(data.callback) === 'function') { data.callback() }
    }).catch(error => {
      handleApiError(this, error, 'could not load file log')
      if (data && typeof(data.callbackCancel) === 'function') { data.callbackCancel() }
    })
  },

  updateFile (ctx, data) {
    let uri = process.env.VUE_APP_API_TANK + 'shows/' + data.show + '/files/' + data.file
    // TODO: add mechanism to indicate the running patch request in the files table
    axios.patch(uri, data.metadata, {
      withCredentials: true,
      headers: { 'Authorization': 'Bearer ' + ctx.rootState.auth.user.access_token }
    }).then(response => {
      ctx.commit('setFileMeta', {id: data.file, metadata: response.data.metadata})
      if (data && typeof(data.callback) === 'function') { data.callback() }
    }).catch(error => {
      handleApiError(this, error, 'could not update file')
      if (data && typeof(data.callbackCancel) === 'function') { data.callbackCancel() }
    })

  },

}

export default {
  namespaced: true,
  state,
  getters,
  actions,
  mutations,
}
